"""
llama-benchy - llama-bench style benchmarking tool for all backends

This package provides a benchmarking tool for OpenAI-compatible LLM endpoints,
generating statistics similar to `llama-bench`.
"""

from ._version import __version__
